//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication

typealias ClientInitializerHandler = (MobileAuthenticationClient) -> ()

struct InitClientMethodHandler {
	func execute(with message: ChannelInMessage, result: @escaping FlutterResult, completion handler: @escaping ClientInitializerHandler) throws {
		guard let message = message as? InitClientMessage else {
			throw PluginError.illegalArgument("Wrong type of message received!")
		}

		let operation = InitializationOperation(operationId: message.operationId)
		OperationCache.shared.put(operation, using: message.operationId)

		let clientInitializer = MobileAuthenticationClientInitializer()
		if let configuration = message.configuration {
			clientInitializer.configuration(configuration)
		}

		if message.onSuccessProvided {
			clientInitializer.onSuccess { client in
				handler(client)
				operation.onSuccess()
			}
		}

		if message.onErrorProvided {
			clientInitializer.onError(operation.onError)
		}

		clientInitializer.execute()
		result(EmptyResultMessage())
	}
}
