//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import Flutter
import NevisMobileAuthentication
import UIKit

public class NevisMobileAuthenticationSdkFlutterSwift: NSObject {
	// MARK: Properties

	private var client: MobileAuthenticationClient?

	private let methodMessageMap: [ChannelMethod: ChannelInMessage.Type] = [
		.initClient: InitClientMessage.self,
		.payloadDecode: OutOfBandPayloadDecodeMessage.self,
		.oobOperation: OutOfBandOperationMessage.self,
		.oobRegister: OutOfBandRegistrationMessage.self,
		.oobAuthenticate: OutOfBandAuthenticationMessage.self,
		.register: RegistrationMessage.self,
		.authCloudApiRegister: AuthCloudApiRegistrationMessage.self,
		.authenticate: AuthenticationMessage.self,
		.deregister: DeregistrationMessage.self,
		.changePin: PinChangeMessage.self,
		.deviceInformationChange: DeviceInformationChangeMessage.self,
		.cancel: CancelMessage.self,
		.accountUsername: AccountUsernameMessage.self,
		.authenticatorAaid: AuthenticatorAaidMessage.self,
		.pin: PinMessage.self,
		.pins: PinsMessage.self,
		.verifyPin: VerifyPinMessage.self,
		.isPolicyCompliant: IsPolicyCompliantMessage.self,
		.listenForOsCredentials: ListenForOsCredentialsMessage.self,
		.localAccounts: LocalAccountsMessage.self,
		.localAuthenticators: LocalAuthenticatorsMessage.self,
		.localDeviceInformation: LocalDeviceInformationMessage.self,
		.localDeleteAuthenticator: LocalDeleteAuthenticatorMessage.self,
	]
}

// MARK: - Flutter Plugin

extension NevisMobileAuthenticationSdkFlutterSwift: FlutterPlugin {
	public static func register(with registrar: FlutterPluginRegistrar) {
		let channel = FlutterMethodChannel(name: "nevis_mobile_authentication_sdk_ios",
		                                   binaryMessenger: registrar.messenger(),
		                                   codec: FlutterJSONMethodCodec.sharedInstance())
		MethodChannelHandler.shared.channel = channel
		let instance = NevisMobileAuthenticationSdkFlutterSwift()
		registrar.addMethodCallDelegate(instance, channel: channel)
	}

	public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
		guard let method = ChannelMethod(rawValue: call.method),
			  let messageType = methodMessageMap[method]
		else {
			return result(FlutterMethodNotImplemented)
		}

		let message = try! JsonMapper.decode(messageType, from: call.arguments ?? [:] as [String: Any])
        if case .initClient = method {
            try! InitClientMethodHandler().execute(with: message, result: result) { self.client = $0 }
        } else {
            precondition(client != nil, "Mobile Authentication Client was not initialized!")
            let handler = MethodHandlerFactory.create(by: messageType)
            try! handler?.execute(using: client!, with: message, result: result)
        }
	}
}
