// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

/// iOS side implementation of [NevisMobileAuthenticationSdkPlatform].
class NevisMobileAuthenticationSdkIos
    extends NevisMobileAuthenticationSdkPlatform {
  static const MethodChannel _channel = MethodChannel(
    'nevis_mobile_authentication_sdk_ios',
    JSONMethodCodec(),
  );
  final MethodChannelMethodCallHandler _handler;
  final MethodChannelIosOperations _operations =
      MethodChannelIosOperations(_channel);
  final MethodChannelIosUserInteractions _userInteractions =
      MethodChannelIosUserInteractions(_channel);
  final MethodChannelIosOsAuthenticationInteractions
      _osAuthenticationInteractions =
      MethodChannelIosOsAuthenticationInteractions(_channel);
  final MethodChannelIosLocalData _localData =
      MethodChannelIosLocalData(_channel);

  /// Registers this class as the default instance of [NevisMobileAuthenticationSdkPlatform].
  static void registerWith() {
    NevisMobileAuthenticationSdkPlatform.instance =
        NevisMobileAuthenticationSdkIos();
  }

  /// Default constructor for [NevisMobileAuthenticationSdkIos].
  NevisMobileAuthenticationSdkIos()
      : _handler = MethodChannelMethodCallHandler();

  /// Test constructor for [NevisMobileAuthenticationSdkIos].
  @visibleForTesting
  NevisMobileAuthenticationSdkIos.mock(MethodChannelMethodCallHandler handler)
      : _handler = handler {
    _channel.setMethodCallHandler(_handler.methodCallHandler);
  }

  @override
  OperationsPlatformInterface get operations => _operations;

  @override
  UserInteractionsPlatformInterface get userInteractions => _userInteractions;

  @override
  OsAuthenticationListenHandlerPlatformInterface
      get osAuthenticationInteractions => _osAuthenticationInteractions;

  @override
  LocalDataPlatformInterface get localData => _localData;

  @override
  Future<void> initClient({
    required String operationId,
    Configuration? configuration,
    required bool onSuccessProvided,
    Function? onSuccess,
    Function(InitializationError)? onError,
  }) async {
    _channel.setMethodCallHandler(_handler.methodCallHandler);
    final operation = InitializationPlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    final message = InitClientMessage(
      operationId: operationId,
      configuration: configuration,
      onSuccessProvided: onSuccessProvided,
      onErrorProvided: (onError != null),
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.initClient,
      message,
    );
  }
}
