// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

/// iOS side implementation of [UserInteractionsPlatformInterface].
class MethodChannelIosUserInteractions
    extends UserInteractionsPlatformInterface {
  final MethodChannel _channel;

  /// Default constructor for [MethodChannelIosUserInteractions].
  MethodChannelIosUserInteractions(this._channel);

  @override
  Future<void> cancel(String operationId) async {
    final message = OperationIdMessage(
      operationId: operationId,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.cancel,
      message,
    );
  }

  @override
  Future<void> authenticatorAaid(String operationId, String aaid) async {
    final message = AuthenticatorAaidMessage(
      operationId: operationId,
      aaid: aaid,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.authenticatorAaid,
      message,
    );
  }

  @override
  Future<bool> isPolicyCompliant(String operationId, String aaid,
      {String? username}) async {
    Map<String, dynamic> message = IsPolicyCompliantMessage(
      operationId: operationId,
      username: username,
      aaid: aaid,
    ).toJson();
    final Map<String, dynamic> responseMap = await _channel.invokeMethod(
      ChannelMethodNames.authenticatorSelectionContext.isPolicyCompliant,
      message,
    );
    return responseMap.fromMap<bool>().result;
  }

  @override
  Future<void> verifyPin(String operationId, String pin) async {
    Map<String, dynamic> message = VerifyPinMessage(
      operationId: operationId,
      pin: pin,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.verifyPin,
      message,
    );
  }

  @override
  Future<void> listenForOsCredentials(
    String operationId,
    PromptOptions? promptOptions,
  ) async {
    // the iOS plugin doesn't need for biometricOptions
    Map<String, dynamic> message = ListenForOsCredentialsMessage(
      operationId: operationId,
      promptOptions: null,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.listenForOsCredentials,
      message,
    );
  }

  @override
  Future<void> pins(String operationId, String oldPin, String newPin) async {
    Map<String, dynamic> message = PinsMessage(
      operationId: operationId,
      oldPin: oldPin,
      newPin: newPin,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.pins,
      message,
    );
  }

  @override
  Future<void> accountUsername(String operationId, String username) async {
    Map<String, dynamic> message = AccountUsernameMessage(
      operationId: operationId,
      username: username,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.accountUsername,
      message,
    );
  }

  @override
  Future<void> pin(String operationId, String pin) async {
    Map<String, dynamic> message = PinMessage(
      operationId,
      pin,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.userInteraction.pin,
      message,
    );
  }
}
