// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';

import '../test_data/common_test_data.dart';

class MethodChannelMock {
  final MethodChannel methodChannel;
  final String method;
  final dynamic result;
  late dynamic mappedArgs = {};
  final Duration delay;

  MethodChannelMock({
    required this.method,
    this.result,
    this.delay = Duration.zero,
  }) : methodChannel = const MethodChannel(
          CommonTestData.methodChannelName,
          JSONMethodCodec(),
        ) {
    TestDefaultBinaryMessengerBinding.instance!.defaultBinaryMessenger
        .setMockMethodCallHandler(methodChannel, _handler);
  }

  Future _handler(MethodCall methodCall) async {
    if (methodCall.method != method) {
      throw MissingPluginException('No implementation found for method '
          '$method on channel ${methodChannel.name}');
    }

    mappedArgs = methodCall.arguments;
    return Future.delayed(delay, () {
      if (result is Exception) {
        throw result;
      }

      return Future.value(result);
    });
  }
}
