// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:mockito/annotations.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import 'method_channel_test_shared_mocks.mocks.dart';

@GenerateMocks([
  // operations
  PlatformOperation,
  InitializationPlatformOperation,
  UserInteractionPlatformOperation,
  OutOfBandPlatformOperation,
  OutOfBandRegistrationPlatformOperation,
  OutOfBandAuthenticationPlatformOperation,
  OutOfBandPayloadDecodePlatformOperation,
  RegistrationPlatformOperation,
  AuthCloudApiRegistrationPlatformOperation,
  AuthenticationPlatformOperation,
  DeregistrationPlatformOperation,
  DeviceInformationChangePlatformOperation,
  ChangePinPlatformOperation,
  // pin change
  PinChanger,
  PinChangeHandler,
  // enroller
  PinEnroller,
  PinEnrollmentHandler,
  // selector
  AccountSelector,
  AccountSelectionHandler,
  AuthenticatorSelector,
  AuthenticatorSelectionHandler,
  // verifier
  BiometricUserVerifier,
  DevicePasscodeUserVerifier,
  FingerprintUserVerifier,
  PinUserVerifier,
  // authorization
  AuthorizationProvider,
  SessionProvider,
  // others
  DeviceInformation,
  RequestHeaders,
  RetryPolicy,
], customMocks: [
  MockSpec<PlatformOperationCache>(
    as: #MockMethodChannelMethodCallCache,
    fallbackGenerators: {#getOperation: mockGetOperation},
  )
])
void main() {}

T? mockGetOperation<T>(String? operationId) {
  if (T == InitializationPlatformOperation) {
    return MockInitializationPlatformOperation() as T?;
  } else if (T == OutOfBandPlatformOperation) {
    return MockOutOfBandPlatformOperation() as T?;
  } else if (T == OutOfBandRegistrationPlatformOperation) {
    return MockOutOfBandRegistrationPlatformOperation() as T?;
  } else if (T == OutOfBandAuthenticationPlatformOperation) {
    return MockOutOfBandAuthenticationPlatformOperation() as T?;
  } else if (T == OutOfBandPayloadDecodePlatformOperation) {
    return MockOutOfBandPayloadDecodePlatformOperation() as T?;
  } else if (T == RegistrationPlatformOperation) {
    return MockRegistrationPlatformOperation() as T?;
  } else if (T == AuthCloudApiRegistrationPlatformOperation) {
    return MockAuthCloudApiRegistrationPlatformOperation() as T?;
  } else if (T == AuthenticationPlatformOperation) {
    return MockAuthenticationPlatformOperation() as T?;
  } else if (T == UserInteractionPlatformOperation) {
    return MockUserInteractionPlatformOperation() as T?;
  } else if (T == DeregistrationPlatformOperation) {
    return MockDeregistrationPlatformOperation() as T?;
  } else if (T == DeviceInformationChangePlatformOperation) {
    return MockDeviceInformationChangePlatformOperation() as T?;
  } else if (T == ChangePinPlatformOperation) {
    return MockChangePinPlatformOperation() as T?;
  }
  return MockPlatformOperation() as T?;
}
