// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import 'mocks/method_channel_mock.dart';
import 'mocks/method_channel_test_shared_mocks.mocks.dart';
import 'test_data/common_test_data.dart';
import 'test_data/configuration_test_data.dart';
import 'test_data/init_client_test_data.dart';
import 'util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache mockCache;
    late MethodChannelMock channel;

    group("Given registerWith is called", () {
      test(
          'Then NevisMobileAuthenticationSdkPlatform instance is NevisMobileAuthenticationSdkIos',
          () async {
        NevisMobileAuthenticationSdkIos.registerWith();
        expect(
          NevisMobileAuthenticationSdkPlatform.instance,
          isA<NevisMobileAuthenticationSdkIos>(),
        );
      });
    });

    group("Given initClient is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        mockCache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = mockCache;
        channel = MethodChannelMock(
          method: 'initClient',
          result: CommonTestData.simpleBoolSuccessResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When configuration is correct", () {
        test("Then result true is returned", () async {
          final operation = MockInitializationPlatformOperation();
          when(mockCache.createOperation(any)).thenReturn(operation);

          await sdk.initClient(
            operationId: CommonTestData.operationId,
            configuration: ConfigurationTestData.defaultConfiguration,
            onSuccessProvided: true,
            onSuccess: () {},
            onError: (error) {},
          );

          expect(channel.mappedArgs, InitClientTestData.correctArguments);
          verify(mockCache.createOperation(
              argThat(isA<InitializationPlatformOperation>())));
        });
      });
    });
  });
}
