// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_mock.dart';
import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/authorization_provider_test_data.dart';
import '../test_data/common_test_data.dart';
import '../test_data/deregister_test_data.dart';
import '../test_data/request_headers_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late PlatformOperationCache cache;
    late MethodChannelMock channel;
    const methodName = 'deregister';

    group("Given $methodName is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        cache = MockMethodChannelMethodCallCache();
        PlatformOperationCache.instance = cache;
        channel = MethodChannelMock(
          method: methodName,
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the parameters are correct", () {
        group("When the required parameters are correct", () {
          test("Then result is success", () async {
            final operation = MockDeregistrationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);

            await sdk.operations.deregister(
              operationId: CommonTestData.operationId,
              username: CommonTestData.username1,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeregisterTestData.correctArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeregistrationPlatformOperation>(),
              )),
            );
          });
        });

        group("When all parameters are correct", () {
          test("Then result is success", () async {
            final authorizationProvider = MockAuthorizationProvider();
            final requestHeaders = MockRequestHeaders();
            final operation = MockDeregistrationPlatformOperation();
            when(cache.createOperation(any)).thenReturn(operation);
            when(authorizationProvider.toJson()).thenReturn(
              AuthorizationProviderTestData.typedCookieAuthorizationProviderMap,
            );
            when(requestHeaders.toJson()).thenReturn(
              RequestHeadersTestData.requestHeadersMap,
            );

            await sdk.operations.deregister(
              operationId: CommonTestData.operationId,
              username: CommonTestData.username1,
              authorizationProvider: authorizationProvider,
              aaid: CommonTestData.aaid1,
              requestHeaders: requestHeaders,
              onSuccess: () {},
              onError: (error) {},
            );

            expect(
              channel.mappedArgs,
              DeregisterTestData.correctAllArguments,
            );
            verify(
              cache.createOperation(argThat(
                isA<DeregistrationPlatformOperation>(),
              )),
            );
          });
        });
      });
    });
  });
}
