// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'authenticator_test_data.dart';
import 'common_test_data.dart';
import 'pin_authenticator_protection_status_test_data.dart';

class VerifyUserTestData {
  static final typedPinContextMap = {
    'operationId': CommonTestData.operationId,
    'authenticator': AuthenticatorTestData.typedAuthenticator1Map,
    'authenticatorProtectionStatus':
        PinAuthenticatorProtectionStatusTestData.typedLastAttemptFailedMap,
    'lastRecoverableError': {
      'type': 'InvalidPin',
      'data': {'description': 'Current pin was invalid.'}
    }
  };

  static final verifyPinCorrectArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'PinUserVerificationContext',
      'data': typedPinContextMap,
    },
  };

  static final verifyPinWrongArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'PinUserVerificationContext',
      'data': {
        'operationId': CommonTestData.operationId,
        'authenticator': {
          'operationId': CommonTestData.operationId,
          'aaid': CommonTestData.aaid1,
          'isSupportedByHardware': true,
          'registration': {'registeredAccounts': []},
          'userEnrollment': {
            'type': 'SdkUserEnrollment',
            'data': {'enrolledAccounts': []},
          },
        },
        'authenticatorProtectionStatus': {
          'type': 'LastAttemptFailed',
          'data': {
            'remainingRetries': 4,
            'coolDownTimeInSec': 30,
          }
        },
        'lastRecoverableError': {
          'type': 'UnknownErrorType',
          'data': {'description': 'Current pin was invalid.'}
        }
      },
    },
  };

  static final typedFingerprintContextMap = {
    'operationId': CommonTestData.operationId,
    'authenticator': {
      'aaid': CommonTestData.aaid2,
      'isSupportedByHardware': true,
      'registration': {
        'registeredAccounts': [
          {
            'username': CommonTestData.username1,
            'server': {'baseUrl': CommonTestData.baseUrl1}
          },
        ],
      },
      'userEnrollment': {
        'type': 'OsUserEnrollment',
        'data': {'isEnrolled': true},
      }
    },
    'lastRecoverableError': {
      'description': 'Current credentials were invalid.',
      'message': 'Try again!'
    }
  };

  static final verifyFingerprintCorrectArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'FingerprintUserVerificationContext',
      'data': typedFingerprintContextMap,
    }
  };

  static final verifyFingerprintWrongArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'FingerprintUserVerificationContext',
      'data': {
        'operationId': CommonTestData.operationId,
        'authenticator': {
          'operationId': CommonTestData.operationId,
          'aaid': CommonTestData.aaid2,
          'isSupportedByHardware': true,
          'registration': {'registeredAccounts': []},
          'userEnrollment': {
            'type': 'UnknownUserEnrollment',
            'data': {'isEnrolled': true},
          },
        },
        'lastRecoverableError': {
          'description': 'Current credentials were invalid.',
          'message': 'Try again!'
        }
      },
    },
  };

  static final typedBiometricContextMap = {
    'operationId': CommonTestData.operationId,
    'authenticator': {
      'operationId': CommonTestData.operationId,
      'aaid': CommonTestData.aaid3,
      'isSupportedByHardware': true,
      'registration': {
        'registeredAccounts': [
          {
            'username': CommonTestData.username1,
            'server': {'baseUrl': CommonTestData.baseUrl1}
          },
        ],
      },
      'userEnrollment': {
        'type': 'OsUserEnrollment',
        'data': {'isEnrolled': true},
      }
    }
  };

  static final verifyBiometricCorrectArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'BiometricUserVerificationContext',
      'data': typedBiometricContextMap,
    }
  };

  static final verifyBiometricWrongArguments = {
    'operationId': CommonTestData.operationId,
    'context': {
      'type': 'FingerprintUserVerificationContext',
      'data': {
        'operationId': CommonTestData.operationId,
        'authenticator': {
          'operationId': CommonTestData.operationId,
          'aaid': CommonTestData.aaid3,
          'isSupportedByHardware': true,
          'registration': {'registeredAccounts': []},
          'userEnrollment': {
            'type': 'UnknownUserEnrollment',
            'data': {'isEnrolled': true},
          },
        },
        'lastRecoverableError': {
          'description': 'Current credentials were invalid.',
          'message': 'Try again!'
        }
      },
    },
  };

  static final wrongIdArguments = {
    'operationId': CommonTestData.wrongId,
    'context': {
      'type': 'PinUserVerificationContext',
      'data': {
        'operationId': CommonTestData.operationId,
        'authenticator': {
          'operationId': CommonTestData.operationId,
          'aaid': CommonTestData.aaid1,
          'isSupportedByHardware': true,
          'registration': {'registeredAccounts': []},
          'userEnrollment': {
            'type': 'SdkUserEnrollment',
            'data': {'enrolledAccounts': []},
          },
        },
        'authenticatorProtectionStatus': {
          'type': 'LastAttemptFailed',
          'data': {
            'remainingRetries': 4,
            'coolDownTimeInSec': 30,
          }
        },
        'lastRecoverableError': {
          'type': 'InvalidPin',
          'data': {'description': 'Current pin was invalid.'}
        }
      },
    },
  };
}
