// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';

import '../mocks/method_channel_mock.dart';
import '../test_data/common_test_data.dart';
import '../test_data/is_policy_compliant_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late MethodChannelMock channel;

    group("Given isPolicyCompliant is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        channel = MethodChannelMock(
          method: 'isPolicyCompliant',
          result: CommonTestData.simpleBoolSuccessResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the state is correct", () {
        test("Then void result is returned", () async {
          final result = await sdk.userInteractions.isPolicyCompliant(
            CommonTestData.operationId,
            CommonTestData.aaid1,
          );
          expect(
              channel.mappedArgs, IsPolicyCompliantTestData.correctArguments);
          expect(result, true);
        });
      });
    });
  });
}
