// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';

import '../mocks/method_channel_mock.dart';
import '../test_data/biometric_prompt_options_test_data.dart';
import '../test_data/common_test_data.dart';
import '../test_data/listen_for_os_credentials_test_data.dart';
import '../util/ios_test_init.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late NevisMobileAuthenticationSdkIos sdk;
    late MethodChannelMock channel;

    group("Given listenForOsCredentials is called", () {
      setUp(() {
        sdk = NevisMobileAuthenticationSdkIos();
        channel = MethodChannelMock(
          method: 'listenForOsCredentials',
          result: CommonTestData.emptyResponse,
        );
      });

      tearDown(() {
        resetMockitoState();
      });

      group("When the state is correct", () {
        test("Then void result is returned", () async {
          await sdk.userInteractions.listenForOsCredentials(
            CommonTestData.operationId,
            BiometricPromptOptionsTestData.options,
          );
          expect(
            channel.mappedArgs,
            ListenForOsCredentialsTestData.correctArguments,
          );
        });
      });
    });
  });
}
